/* LASCAR (LibrAirie Socle CARtographie)
 * Version     : 0.2.2
 * Auteur      : Loïc BRULEBOIS
 * Description :
 * Librairie permettant de générer facilement une carte 
 * à partir des tuiles d'OpenStreetMap. Cette Librairie
 * permet aussi de personnaliser la carte en y intégrant
 * des données GeoJSON ou OverPass.
 */
(function (global, factory) {
    // Pour les environnements de type CommonJS ou NodeJS
    // on intègre la librairie dans les modules 
    if (typeof module == "object" && typeof module.exports == "object") {
        module.exports = factory(global, true);
    }
    // Sinon on se contente d'instancier la librairie ...
    else {
        factory(global);
    }

})(typeof window !== "undefined" ? window : this, function (global, noGlobal) {
    "use strict";

    // On vérifie que la librairie JQuery est
    // initialisée sans quoi on retourne une erreur !
    if (typeof $ == "undefined") {
        throw new Error("The jQuery library is required.");
    }

    // On vérifie que la librairie Leaflet est
    // initialisée sans quoi on retourne une erreur !
    if (typeof L == "undefined") {
        throw new Error("The Leaflet library is required.");
    }

    /** Constructeur d'une instance de la librairie. */
    function Lascar() {
        this.version = "0.2.2";
    };



    // ----------
    // FONCTIONS
    // ----------

    /* Domaine permettant de contenir les fonctions utiles
     * apportées par la librairie. 
     */
    Lascar.prototype.fn = {};

    /* Méthode permettant d'intégrer des variables 
     * dans une chaîne de caractères à partir d'indices.
     */
    Lascar.prototype.fn.strFormat = function (str) {
        // On vérifie que le premier argument est bien
        // une chaîne de caractères.
        if (typeof str !== "string" || str == null) {
            throw new Error("format() : the function needs a valid string input.");
        }

        // On commence par récupérer les potentiels
        // arguments permettant de replacer les balises
        // {x} (avec x un nombre entier > à 0) par
        // l'argument approprié.
        var args = (function (args) {
            var _args = new Array();
            $.each(args, function (i, arg) {
                if (i > 0) {
                    _args.push(arg);
                }
            });
            return _args;
        })(arguments);

        // On peut enfin remplacer les balises dans
        // la chaîne de caractères fournie ...
        return str.replace(/{(\d+)}/g, function (match, value) {
            // Si aucun argument n'existe pour cet indice, on
            // retourne une erreur !
            if (value >= args.length) {
                throw new Error("format() : no arguments found for the index ".concat(value).concat("."));
            }
            // Sinon on retourne la valeur associée !
            return args[value];
        });
    };
    // On intègre cette fonction dans la classe String
    // pour faciliter son usage !
    String.prototype.format = function () {
        return Lascar.prototype.fn.strFormat.apply(this, (function (src, args) {
            var _args = new Array();
            _args.push(src);
            $.each(args, function (i, arg) {
                _args.push(arg);
            });
            return _args;
        })(this, arguments));
    };

    /* Méthode permettant de compter le nombre d'élément
     * correspondant au critère de tri issu d'une fonction
     * par rapport à un tableau.
     */
    Lascar.prototype.fn.arrCount = function (arr, sorter) {
        // On vérifie que le premier argument est bien
        // un tableau, si le deuxième est une fonction,
        // on l'exploitera plus tard en tant que tel !
        if (!(arr instanceof Array)) {
            throw new Error("count() : the function needs a valid array.");
        }

        // On part d'un objet vide, permettant de contenir 
        // la classification des éléments à partir de la
        // fonction de tri. Cet objet est représenté dans la
        // méthode "reduce()" par l'attribut "prevVal".
        return arr.reduce(function (prevVal, current, index, array) {
            var sortResult = (sorter || String)(current);
            prevVal[sortResult] = prevVal.hasOwnProperty(sortResult) ? ++prevVal[sortResult] : 1;
            return prevVal;
        }, {});
    };
    // On intègre cette fonction dans la classe Array
    // pour faciliter son usage !
    Array.prototype.count = function (sorter) {
        return Lascar.prototype.fn.arrCount(this, sorter);
    };

    /* Méthode permettant de récupérer le tableau 
     * différentiel entre un tableau source et un 
     * tableau de destination.
     */
    Lascar.prototype.fn.arrDiff = function (arrSrc, arrCmp) {
        // On vérifie que le premier argument est bien
        // un tableau et que la fonction de trie est
        // bien une fonction.
        if (!(arrSrc instanceof Array) || !(arrCmp instanceof Array)) {
            throw new Error("format() : the function needs two valid arrays.");
        }

        // On cherche a récupérer l'ensemble des valeurs
        // qui sont dans le tableau source mais pas dans
        // le tableau de comparaison.
        return arrSrc.filter(function (current, index, array) {
            return $.inArray(current, arrCmp) < 0;
        });
    };
    // On intègre cette fonction dans la classe Array
    // pour faciliter son usage !
    Array.prototype.diff = function (array) {
        return Lascar.prototype.fn.arrDiff(this, array);
    };

    /* Méthode permettant de vérifier qu'une valeur
     * est bien un entier.
     */
    Lascar.prototype.fn.numIsInt = function (value) {
        // Source : https://developer.mozilla.org/fr/docs/Web/JavaScript/Reference/Objets_globaux/Number/isInteger
        return typeof value == "number" && ifFinite(value) && Math.floor(value) === value;
    };
    // On intègre cette fonction dans la classe Number
    // pour faciliter son usage (si nécessaire) !
    Number.isInteger = Number.isInteger || Lascar.prototype.fn.numIsInt;

    /* Méthode permettant de récupérer le nom d'un
     * constructeur dans le cadre de la vérification
     * des types.
     */
    Lascar.prototype.fn.getConstructorName = function(constructor) {
        var name = constructor.name;
        // Pour certains navigateurs (IE), le nom du constructeur 
        // est undefined, pour contrer cela, nous devons le
        // récupérer "manuellement" ...
        if(typeof name == "undefined") {
            var match = undefined;
            // On récupère le nom de la fonction à partir
            // de sa description ...
            if((match = constructor.toString().match(/^function\s+([\w\$]+)\s*\(/))) {
                name = match[1];
            } 
            // En cas d'échec, on retourne "Function".
            else {
                name = "Function";
            }
        }
        // On retourne enfin le résultat de la recherche.
        return name;
    };

    /* Méthode permettant de vérifier l'intégrité d'un 
     * objet par rapport à un pattern (type ou descriptif
     * de l'objet).
     */
    Lascar.prototype.fn.check = function (obj, pattern) {
        // Si le pattern est une chaîne de caractères ...
        if (typeof pattern == "string") {
            var matcher = null;
            // Dans le cas d'une description de type "x[]", 
            // on doit s'assurer que l'objet est tant une 
            // instance de Array que le type des objets qu'il
            // contient corresponde au type décrit par "x".
            if ((matcher = pattern.match(/^([a-zA-Z0-9_-]+)\[\]$/))) {
                // Si l'objet n'est pas un tableau, on retourne false.
                if (!(obj instanceof Array)) { return false; }

                // Enfin, pour chaque élément du tableau, on 
                // vérifie que son type correspond à un type 
                // attendu !
                var itemsAreValid = true;
                $.each(obj, function (i, param) {
                    if (!Lascar.prototype.fn.check(param, matcher[1])) {
                        itemsAreValid = false;
                    }
                });

                return itemsAreValid;
            }
            // S'il s'agît d'un type primitif, on se contente 
            // de vérifier l'objet avec un typeof !
            else if (pattern.match(/(string|number|function|object|undefined|boolean)/)) {
                return typeof obj === pattern;
            }
            // Cas particulier du pattern décrivant un objet
            // "null", on doit le vérifier à part ...
            else if (pattern.match(/null/)) {
                return obj === null;
            }
            // Enfin, s'il s'agît d'une chaîne de caractère ne 
            // correspondant pas à un type primitif, on vérifie
            // que le nom d'un des constructeurs de l'objet 
            // corresponde au type décrit !
            else {
                // S'il l'objet n'en est pas un ou est null, 
                // on retourne false directement !
                if (typeof obj != "object" || obj == null) { return false; }

                var cstrToCk = obj.constructor,
                    cstrName = Lascar.prototype.fn.getConstructorName(cstrToCk);
                // Tant qu'il ne s'agît pas d'un type primitif 
                // "Function" ou "Object", on cherche à trouver
                // le constructeur dont le nom matchera avec celui
                // fourni en paramètre.
                while (cstrName !== "Function" && cstrName !== "Object") {
                    if (cstrName.match(pattern)) { return true; }
                    cstrToCk = cstrToCk.constructor;
                }

                // Si aucun constructeur ne correspond, on retourne 
                // false ...
                return false;
            }
        }
        // Si le pattern est un tableau d'instances décrivant 
        // les paramètres d'un objet ...
        else if (Lascar.prototype.fn.check(pattern, "Description[]")) {
            var isValid = true;
            // Pour chaque élément obligatoire dans l'objet ...
            $.each(pattern, function (i, control) {
                // Si l'objet ne dispose pas de la propriété, on
                // retourne false.
                if (!obj.hasOwnProperty(control.label)) { isValid = false; }

                // Si la propriété n'est pas du type attendu, on
                // retourne false aussi !
                if (control.type instanceof Array) {
                    var matchOnce = false;
                    $.each(control.type, function (i, type) {
                        if (Lascar.prototype.fn.check(obj[control.label], type)) {
                            matchOnce = true;
                        }
                    });

                    if (!matchOnce) { isValid = false; }
                } else if (!Lascar.prototype.fn.check(obj[control.label], control.type)) {
                    isValid = false;
                }
            });
            return isValid;
        }
        // Si le pattern est une fonction, on vérifie la 
        // correspondance à l'aide de la fonction "instanceof" 
        else if (typeof pattern == "function") {
            // Cas particulier pour une classe de type "ArrayOf",
            // on vérifie aussi au travers du 3ème argument décirvant
            // le type des objets du tableau (si fourni).
            if (typeof arguments[2] == "function") {
                return obj instanceof pattern && obj.getType() === arguments[2];
            } else {
                return obj instanceof pattern;
            }
        }
        // Sinon, on retourne une erreur.
        else {
            throw new Error("check() needs a valid pattern description.");
        }
    };

    /* Méthode permettant de corriger le chemin des 
     * ressources nécessaires à Leaflet lorsque le chemin
     * n'a pas pu être détecté automatiquement.
     */
    Lascar.prototype.fn.lfCorrect = function (path) {
        // S'il s'agît d'un RegExp, on cherche parmis tous
        // les chemins CSS, celui qui correspond au pattern 
        if (path instanceof RegExp) {
            // Pour chaque feuille de style dans la page,
            // on cherche celle qui correspond au pattern
            // pour récupérer le chemin et y ajouter 'images/'
            $("link[rel='stylesheet']").each(function (i, s) {
                if (s.href.match(path)) {
                    L.Icon.Default.imagePath = "{0}images/".format(s.href.split(path)[0]);
                    return true;
                }
            });
        }
        // S'il s'agît d'une chaîne de caractères, cela 
        // signifie que l'utilisateur nous a fourni directement
        // le chemin à utiliser 
        else if (typeof path == "string") {
            L.Icon.Default.imagePath = path;
        }
        // Sinon on retourne une erreur signifiant que l'on
        // ne sait pas traiter l'élément fourni par l'utilisateur.
        else {
            throw new Error("lfCorrect() needs a valid RegExp or a string.");
        }
    };
    // On intègre cette fonction dans la rubrique Util
    // pour faciliter son usage !
    L.Util = $.extend({
        correctPath: Lascar.prototype.fn.lfCorrect
    }, L.Util);

    /* Méthode permettant d'étendre les propriétés d'un
     * objet à l'aide d'un objet complémentaire. Si
     * une proriété de l'objet complémentaire a pour 
     * valeur "undefined" et que ce même paramètre est
     * défini dans l'objet source, la propriété est
     * supprimée de l'objet source.
     */
    Lascar.prototype.fn.extend = function (src, cmp) {
        // Pour cette méthode, l'objet source doit forcément
        // exister et être non null, de même pour l'objet 
        // complémentaire !
        if (typeof src != "object" || typeof cmp != "object" || src == null || cmp == null) {
            throw new Error("extend() needs two valid objects.");
        }

        // On clône la source pour éviter de l'impacter
        // définitivement.
        var source = $.extend(true, {}, src);
        var deleteUndefined = function (obj, pattern) {
            $.map(pattern, function (val, key) {
                switch (typeof val) {
                    case "undefined":
                        if (obj.hasOwnProperty(key)) {
                            delete obj[key];
                        }
                        break;

                    case "object":
                        if (obj.hasOwnProperty(key)) {
                            deleteUndefined(obj[key], val);
                        }
                        break;
                }
            });
        };
        // On commence par supprimer de l'objet source 
        // l'ensemble des paramètres nouvellement défini
        // à undefined.
        deleteUndefined(source, cmp);

        // Puis on continue par l'extension de l'objet  
        // source avec l'objet complémentaire ...
        return $.extend(true, source, cmp);
    };

    /* Méthode permettant de déplier un objet de manirère 
     * à retourner chaque propriété (quel que soit le niveau)
     * avec sa définition complète.
     */
    Lascar.prototype.fn.unfold = function (obj) {
        var unfoldObject = function (src, root, result) {
            // Pour chaque propriété de l'objet ...
            $.map(src, function (val, key) {
                // On génère l'identifiant complet à partir 
                // de la racine si elle est définie.
                var id = "{0}{1}".format(
                    typeof root == "string" ? "{0}.".format(root) : "",
                    key
                );

                // S'il s'agît d'un objet non null, on le 
                // parcourt ...
                if (typeof val == "object" && val != null) {
                    unfoldObject(val, id, result);
                }
                // Sinon on intègre la valeur ...
                else {
                    result[id] = val;
                }
            });

            return result;
        };
        return unfoldObject(obj, null, {});
    };



    // ----------
    // CLASSES
    // ----------

    /* Domaine permettant de contenir les classes apportées
     * par la librairie.
     */
    Lascar.prototype.cl = {};

    /* Domaine permettant de contenir les classes dîtes 
     * utiles au bon fonctionnement de la librairie.
     */
    Lascar.prototype.cl.util = {

        /* Classe permettant de générer un tableau tout en
         * spécifiant le type des objets attendus. Utile
         * pour la vérification par la suite.
         */
        ArrayOf: (function () {
            function ArrayOf(type) {
                // Le paramètre 'type' doit forcément être une
                // fonction !
                if (typeof type != "function") {
                    throw new Error("ArrayOf needs a valid type definition.");
                }

                var type = type;
                /* Méthode permettant de récupérer le type
                 * des objets attendus dans le tableau.
                 */
                this.getType = function () {
                    return type;
                };

                // On vérifie si d'autres arguments sont disponibles,
                // cela signifie que l'utilisateur souhaite initialiser
                // l'objet à partir de ces éléments ...
                if (arguments.length > 1) {
                    var _this = this;
                    $.each(arguments, function (i, arg) {
                        if (i >= 1) {
                            _this.push(arg);
                        }
                    });
                }
            };

            // Héritage de la classe "Array"
            ArrayOf.prototype = Object.create(Array.prototype);
            ArrayOf.prototype.constructor = ArrayOf;

            /* Méthode permettant d'ajouter un ou plusieurs éléments
             * au tableau tout en contrôlant que le type soit bien 
             * celui attendu.
             */
            ArrayOf.prototype.push = function () {
                var _this = this;
                // Pour chaque paramètre, ...
                $.each(arguments, function (i, arg) {
                    // S'il s'agît d'un tableau, pour chaque 
                    // élément de celui-ci, on l'intègre !
                    if (arg instanceof Array) {
                        $.each(arg, function (i, item) {
                            _this.push(item);
                        });
                    }
                    // Sinon, on s'assure que l'argument
                    // soit du bon type !
                    else {
                        if (!(arg instanceof _this.getType())) {
                            throw new Error("ArrayOf needs a valid object waited type.");
                        }
                        Array.prototype.push.call(_this, arg);
                    }
                });
            };

            return ArrayOf;
        })(),

        /* Classe permettant de générer la description d'un
         * paramètre d'un objet.
         */
        Description: (function () {
            function Description(label, type) {
                // On vérifie que les paramètres fournis soient valides !
                if (typeof label != "string" || !label.match(/([a-zA-Z0-9_-]+)/)) {
                    throw new Error("The Description needs a valid string label.");
                }

                if (!(typeof type).match(/(string|function|object)/) && !(type instanceof Lascar.prototype.cl.util.Description)) {
                    throw new Error("The Description needs a valid type.");
                }

                this.label = label;
                this.type = type;
            };

            return Description;
        })(),

        /* Classe permettant de générer un objet observable
         * de manière à respecter le pattern Observateur/Observable.
         */
        Observable: (function () {
            /* Constructeur d'un nouvel objet observable. 
             */
            function Observable() {
                var
                    // Liste des objets observant l'instance.
                    observers = new Array(),
                    // Indicateur permettant de définir si 
                    // l'état de l'objet a changé ou non.
                    changed = false;

                /* Méthode permettant de récupérer la liste
                 * des observateurs.
                 */
                this.getObservers = function () {
                    return $.extend(true, new Array(), observers);
                };

                /* Méthode permettant de récupérer l'état de 
                 * l'indicateur de changement de statut de
                 * l'instance.
                 */
                this.hasChanged = function () {
                    return changed;
                };

                /* Méthode permettant d'ajouter un ou plusieurs
                 * objets en tant qu'observateur de l'instance.
                 */
                this.addObservers = function (obs) {
                    // Si le paramètre est un tableau, on ajoute
                    // l'ensemble des observateurs.
                    if (obs instanceof Array) {
                        var _this = this;
                        $.each(obs, function (i, o) {
                            _this.addObservers(o);
                        });
                    }
                    // Sinon, on s'assure que l'observateur est bien
                    // un objet qui n'est pas déjà référencé.
                    else if (typeof obs == "object" && obs != null && $.inArray(obs, observers) == -1) {
                        observers.push(obs);
                    }
                };

                /* Méthode permettant de mettre-à-jour l'indicateur 
                 * de l'état de changement de l'instance. 
                 */
                this.setChanged = function (val) {
                    changed = val;
                };
            };

            /* Méthode permettant de notifier les observateurs 
             * d'un changement d'état de l'instance avec une 
             * donnée embarquée.
             */
            Observable.prototype.notify = function (data) {
                // Si l'objet actuel n'a pas subi de
                // changement, inutile de notifier les 
                // observateurs !
                if (!this.hasChanged()) { return false; }

                var _this = this;
                // Pour chaque observateur ...
                $.each(this.getObservers(), function (i, o) {
                    // Si l'observateur possède une méthode 
                    // 'update(obs, d)', alors on le notifie !
                    if (typeof o == "object" && o != null && typeof o.update == "function") {
                        o.update.call(o, _this, typeof data == "undefined" ? null : data);
                    }
                });

                // Une fois l'ensemble des observateurs notifiés,
                // on rebascule l'indicateur de changement à false.
                this.changed = false;

                return true;
            };

            return Observable;
        })(),

        /* Classe permettant de générer un sémaphore de
         * manière à protéger les sections critiques.
         */
        Semaphore: (function () {
            /* Constructeur d'un nouveau sémaphore à partir
             * d'un nombre maximum de "jetons". 
             */
            function Semaphore(max) {
                // Si 'max' n'est pas un nombre positif ou null,
                // on retourne une erreur !
                if (typeof max != "number" || !Number.isInteger(max)) {
                    throw new Error("Semaphore() needs an integer as first parameter.");
                }

                // Propriétés privées.
                var queue = new Array(),
                    givenTokens = 0,
                    _this = this;
                /* Méthode permettant d'accéder à la ressource 
                 * ou d'inscrire l'action à réaliser dans
                 * la pile des actions à suivre.
                 */
                this.acquire = function (fn) {
                    // S'il y a des jetons disponibles,
                    // on exécute directement l'action
                    // tout en récupérant un jeton ...
                    if (givenTokens < max) {
                        givenTokens++;
                        dispatch(fn);
                    }
                    // Sinon on la référence ...
                    else {
                        queue.push(fn);
                    }
                };
                /* Méthode permettant de libérer une place pour 
                 * permettre à l'action suivante de se 
                 * réaliser. */
                this.release = function () {
                    var next = queue.shift();
                    // S'il n'y a plus aucune action à 
                    // réaliser, on libère un jeton ...
                    if (typeof next == "undefined") {
                        // On s'assure toutefois que le nombre
                        // de jetons distribués ne descende pas
                        // en dessous de zéro !
                        if (givenTokens > 0) {
                            givenTokens--;
                        }
                    }
                    // Sinon on exécute l'action suivante ...
                    else {
                        dispatch(next);
                    }
                };

                /* Méthode permettant d'exécuter une seule et unique
                 * fois l'action demandée tout en intégrant 
                 * une méthode de libération du token à exécuter.
                 */
                var dispatch = function (fn) {
                    var executed = false;
                    fn(function () {
                        if (executed === false) {
                            executed = true;
                            _this.release();
                        }
                    })
                };
            };

            return Semaphore;
        })()

    };

    /* Domaine permettant de contenir les classes 
     * de génération des layers pour une carte 
     * générée par Lascar.
     */
    Lascar.prototype.cl.layer = {

        /* Classe permettant de générer un layer exploitant
         * des tuiles de type Raster et faciliant la génération
         * des composants sur une carte Leaflet.
         */
        RasterTileLayer: L.TileLayer.extend({
            /* Paramètre permettant de définir les options
             * par défaut d'un layer de type Raster.
             */
            default: {
                // Origine de la source des tuiles
                "attribution": "&copy; contributeurs d'<a href=\"http://openstreetmap.org\">OpenStreetMap</a>",
                // Zoom maxmimal
                "maxZoom": 18,
                // Zoom minimal
                "minZoom": 0,
                // Indicateur de layer par défaut
                "default": false
            },

            /* Constructeur d'un layer de type raster permettant
             * de le générer avec des options personnalisées et/ou
             * celles par défaut.
             */
            initialize: function (url) {
                // On vérifie que l'URL est bien une chaîne de
                // caractères non vide !
                if (typeof url != "string" || $.trim(url) == "") {
                    throw new Error("RasterTileLayer needs a valid string URL.");
                }

                // On récupère les options à appliquer ...
                var options = arguments.length >= 2 && typeof arguments[1] == "object" && arguments[1] != null ?
                    Lascar.prototype.fn.extend(this.default, arguments[1]) :
                    this.default;

                // On génère enfin l'instance à l'aide du contrôleur
                // de base.
                L.TileLayer.prototype.initialize.call(this, url, options);
                // Si l'utilisateur n'a pas défini de titre
                // on en génère un !
                if (typeof this.options.title != "string" || $.trim(this.options.title) == "") {
                    this.options.title = "Raster #{0}".format(L.stamp(this));
                }
                // Même principe pour le code d'identification
                // du layer ...
                if (typeof this.options.code != "string" || $.trim(this.options.code) == "") {
                    this.options.code = "raster_{0}".format(L.stamp(this));
                }
            }
        }),

        /* Classe permettant de générer un layer pouvant être associé
         * à plusieurs cartes. Il est identifiable par le code l'assoçiant
         * à celle-ci.
         */
        FeaturesLayer: L.FeatureGroup.extend({

            /* Constructeur du layer.
             */
            initialize: function () {
                this.codes = {};
                // On prend en compte le/les potentiels
                // paramètres associés.
                L.FeatureGroup.prototype.initialize.apply(this, arguments);
            },

            /* Méthode permettant d'ajouter ou de mettre-à-jour
             * le layer auprès de la carte désignée tout en ajoutant
             * un nom dans le contrôleur de layers de la carte (si
             * celui-ci existe).
             */
            addTo: function (code, map, name, add2ctrl) {
                var dataMap = $.map(this.codes, function (data, code) {
                    return [code, data];
                });
                // On vérifie que le layer n'est pas déjà présent
                // sur la carte sans quoi on se contente de mettre
                // à jour si nécessaire le code.
                if (dataMap.length > 0) {
                    delete this.codes[dataMap[0]];
                    delete map.options.cpl_layers[dataMap[0]];

                    // Si la carte dispose d'un composant de gestion
                    // des layers "layersControl", on supprime l'ancienne
                    // référence pour mieux le référencer à nouveau.
                    if (typeof map.layersControl != "undefined") {
                        map.layersControl.removeLayer(this);
                    }
                }
                // Sinon on référence la carte et on ajoute le 
                // layer.
                else {
                    L.FeatureGroup.prototype.addTo.call(this, map);
                }

                // Si la carte dispose d'un composant de gestion
                // des layers "layersControl", on référence le
                // layer dedans !
                if (add2ctrl && typeof map.layersControl != "undefined") {
                    map.layersControl.addOverlay(this, name || code);
                }

                this.codes[code] = map;
                map.options.cpl_layers[code] = this;
            }

        })

    };

    Lascar.prototype.cl.layer = $.extend(true, Lascar.prototype.cl.layer, {

        /* Classe permettant de générer un layer exploitant 
         * des ressources distantes issues par exemple de
         * web services.
         */
        UrlBasedLayer: Lascar.prototype.cl.layer.FeaturesLayer.extend({
            /* Paramètre permettant de définir les options
             * par défaut du layer.
             */
            default: {
                // Indicateur sur la création automatisée 
                // du composant "Notifier" sur les cartes liées
                // à ce composant.
                autobuildNotifier: true,
                // On définit les styles par défaut des 
                // types d'objets (point, ligne, forme).
                styles: {
                    point: {
                        opacity: 1.0,
                        weight: 2,
                        color: "#0088CE",
                        radius: 5
                    },
                    ligne: {
                        opacity: 1.0,
                        weight: 2,
                        color: "#0088CE"
                    },
                    forme: {
                        opacity: 1.0,
                        weight: 2,
                        color: "#0088CE"
                    }
                }
            },

            /* Constructeur du layer.
             */
            initialize: function (reqConfig) {
                // On génère le requêteur qui va nous être 
                // utile pour récupérer le contenu qui nous 
                // intéresse.
                this.req = new Lascar.prototype.cl.component.Requester(reqConfig);

                // On génère la réaction par défaut suite à une 
                // réponse ou un changement d'état du web service : 
                //   - Dans le cas de l'envoi de la requête
                this.req.on("sending", function (event) {
                    this.notify(
                        "Récupération des données ...",
                        $("<i>", {
                            class: "rotation fa fa-refresh",
                        })[0]
                    );
                }, this);
                //   - Dans le cas d'une réponse en succès
                this.req.on("success", function (event) {
                    this.notify(
                        "Données récupérées ...",
                        $("<i>", {
                            class: "alert fa fa-check"
                        })[0]
                    );
                }, this);
                //   - Dans le cas d'une réponse en erreur
                this.req.on("fail", function (event) {
                    this.notify(
                        "Erreur lors de la récupération des données.",
                        $("<i>", {
                            class: "alert fa fa-exclamation-triangle",
                        })[0]
                    );
                }, this);

                // On prend en compte le/les potentiels
                // paramètres associés.
                Lascar.prototype.cl.layer.FeaturesLayer.prototype.initialize.apply(this, (function (args) {
                    var result = new Array();
                    $.each(args, function (i, value) {
                        if (i > 0) { result.push(value); }
                    });
                    return result;
                })(arguments));

                // On intègre les options par défaut dans les
                // propriétés du layer ...
                this.options = Lascar.prototype.fn.extend(this.options, this.default);
            },

            /* Méthode permettant d'ajouter ou de mettre-à-jour
             * le layer auprès de la carte désignée tout en ajoutant
             * un nom dans le contrôleur de layers de la carte (si
             * celui-ci existe) et si souhaité, générer le composant
             * en charge des notifications.
             */
            addTo: function (code, map, name, add2ctrl) {
                // Si l'options "autobuildNotifier" est activée, on
                // génère si nécessaire sur la carte le composant.
                if (this.options.autobuildNotifier && typeof map.notifierControl == "undefined") {
                    map.notifierControl = new Lascar.prototype.cl.component.Notifier();
                    map.addControl(map.notifierControl);
                }

                // Enfin on appelle la méthode de base ...
                Lascar.prototype.cl.layer.FeaturesLayer.prototype.addTo.apply(this, arguments);
            },

            /* Méthode permettant de notifier la/les cartes associées
             * d'un changement survenu dans la mise-à-jour du layer.
             */
            notify: function (message, icon) {
                var _this = this;
                $.map(this.codes, function (map, code) {
                    // Si la carte dispose d'un composant de notification,
                    // on en profite !
                    if (typeof map.notifierControl != "undefined" && _this.options.autobuildNotifier) {
                        // Si le message est null, cela signifie que
                        // l'on souhaite masquer le notifieur ...
                        if (message == null) {
                            map.notifierControl.hide();
                        } else {
                            map.notifierControl.show({
                                message: message,
                                icon: icon
                            });
                        }
                    }
                });
            },

            /* Méthode permettant de parser les propriétés d'un
             * objet géométrique dans une popup pour faciliter sa
             * compréhension. Cette méthode peut être attacher comme
             * méthode parsing.properties afin que ce traitement 
             * soit effectué automatiquement.
             */
            _parseProperties: function (geometry) {
                if (!(geometry instanceof L.Layer) || typeof geometry.properties != "object" || geometry.properties == null) {
                    throw new Error("parsing.properties() needs a valid geometry object with properties.");
                }

                var content =
                    '<div class="lascar-ct-popup"> \
                       <table class="table"> \
                           <thead> \
                               <tr> \
                                   <th>Clé</th> \
                                   <th>Valeur</th> \
                               </tr> \
                           </thead> \
                           <tbody>';

                $.each(geometry.properties, function (i, v) {
                    content = content.concat(
                        '           <tr> \
                                       <td><code>{0}</code></td> \
                                       <td>{1}</td> \
                                   </tr>'.format(i, v));
                });

                content = content.concat(
                    '       </tbody> \
                       </table> \
                    </div>');

                // Enfin on peut attacher le contenu à l'objet géométrique
                geometry.bindPopup(content);
            }

        })

    });

    Lascar.prototype.cl.layer = $.extend(true, Lascar.prototype.cl.layer, {

        /* Classe permettant de générer un layer exploitant 
         * des ressources distantes issues par exemple de
         * web services retournant des données GeoJSON.
         */
        GeoJSON: Lascar.prototype.cl.layer.UrlBasedLayer.extend({

            /* Constructeur du layer.
             */
            initialize: function (reqConfig) {
                // On commence par appeller le constructeur
                // parent pour initialiser le layer ...
                Lascar.prototype.cl.layer.UrlBasedLayer.prototype.initialize.apply(this, arguments);
                
                // Puis on complète les réactions aux différents
                // statuts de la requête.
                //   - Dans le cas d'une réponse en succès
                //     on parse les données reçues.
                this.req.on("success", function (event) {
                    // On commence par purger le layer actuel ...
                    this.clearLayers();
                    try {
                        // ... puis on peut enfin intégrer les objets
                        // géométriques fournis ...
                        var geoObjects = this.parsing[event.data.type].call(this, event.data);
                        // S'il s'agît d'un tableau, on intègre chacun des objets
                        if (geoObjects instanceof Array) {
                            var _this = this;
                            $.each(geoObjects, function (i, geoObject) {
                                geoObject.addTo(_this);
                            });
                        }
                        // Sinon on intègre directement l'objet !
                        else {
                            geoObjects.addTo(_this);
                        }

                        this.notify(
                            "Parsage terminé !",
                            $("<i>", { class: "alert fa fa-thumbs-up" })[0]
                        );
                    } catch (e) {
                        this.notify(
                            "Erreur lors du parsage des données.",
                            $("<i>", { class: "alert fa fa-exclamation-triangle" })[0]
                        );
                    }
                }, this);
            },

            /* Méthode permettant de convertir les coordonnées
             * en un tableau de coordonnées exploitables.
             */
            _coordinates: function (coordinates) {
                // On s'assure que le paramètre fourni est bien un
                // tableau ... on contrôlera par la suite le contenu
                // de celui-ci !
                if (!(coordinates instanceof Array)) {
                    throw new Error("_coordinates() needs a valid Array as parameter.");
                }

                // Si nous sommes arrivés à un tableau composés 
                // de coordonnées (nombres) on le traite en tant 
                // que tel !
                if (typeof coordinates[0] == "number") {
                    return [coordinates[1], coordinates[0]];
                }
                // Sinon, s'il s'agît d'un tableau composés
                // de tableaux, on les traite récursivement ...
                else if (coordinates[0] instanceof Array) {
                    var result = new Array(),
                        _this = this;
                    $.each(coordinates, function (i, _d) {
                        result.push(_this._coordinates(_d));
                    });
                    return result;
                }
                // S'il ne s'agît pas d'un type traité, on
                // retourne une erreur !
                else {
                    throw new Error("_coordinates() unknown type for coordinates.");
                }
            },

            /* Méthodes de parsage des différentes ressources
             * gérées dans le format GeoJSON pour compléter le
             * layer actuel.
             */
            parsing: {

                /* Méthode dédiée au parsage d'une ressource de type
                 * "Point" issue d'une source GeoJSON.
                 */
                "Point": function (data) {
                    // Si des propriétés de style sont définies, on 
                    // les intègre, sinon on utilise celles par défaut.
                    var style = typeof arguments[1] == "object" && arguments[1] != null ?
                        arguments[1] : this.options.styles.point;
                    // On génère enfin le marqueur ...
                    return new L.circleMarker(this._coordinates(data.coordinates), style);
                },

                /* Méthode dédiée au parsage d'une ressource de type
                 * "LineString" issue d'une source GeoJSON.
                 */
                "LineString": function (data) {
                    // Si des propriétés de style sont définies, on 
                    // les intègre, sinon on utilise celles par défaut.
                    var style = typeof arguments[1] == "object" && arguments[1] != null ?
                        arguments[1] : this.options.styles.ligne;
                    // On génère enfin la ligne ...
                    return new L.polyline(this._coordinates(data.coordinates)).setStyle(style);
                },

                /* Méthode dédiée au parsage d'une ressource de type
                 * "Polygon" issue d'une source GeoJSON.
                 */
                "Polygon": function (data) {
                    // Si des propriétés de style sont définies, on 
                    // les intègre, sinon on utilise celles par défaut.
                    var style = typeof arguments[1] == "object" && arguments[1] != null ?
                        arguments[1] : this.options.styles.forme;
                    // On génère enfin le polygone ...
                    return new L.polygon(this._coordinates(data.coordinates)).setStyle(style);
                },

                /* Méthode permettant de simplifier le traitement des
                 * ressources de type "Multi(*)" issues d'une source GeoJSON.
                 */
                "Multi": function (type, data) {
                    var result = new Array(),
                        _this = this,
                        style = arguments[2];
                    // Pour chaque paquet de coordonnées, on génère les
                    // objets correspondants ...
                    $.each(data.coordinates, function (i, _coordinates) {
                        result.push(_this.parsing[type].call(_this,
                            { coordinates: _coordinates }, style));
                    });
                    // On retourne enfin le fruit du parsage.
                    return result;
                },

                /* Méthode dédiée au parsage d'une ressource de type
                 * "MultiPoint" issue d'une source GeoJSON.
                 */
                "MultiPoint": function (data) {
                    return this.parsing["Multi"].call(this, "Point", data, arguments[1]);
                },

                /* Méthode dédiée au parsage d'une ressource de type
                 * "MultiLineString" issue d'une source GeoJSON.
                 */
                "MultiLineString": function (data) {
                    return this.parsing["Multi"].call(this, "LineString", data, arguments[1]);
                },

                /* Méthode dédiée au parsage d'une ressource de type
                 * "MultiPolygon" issue d'une source GeoJSON.
                 */
                "MultiPolygon": function (data) {
                    return this.parsing["Multi"].call(this, "Polygon", data, arguments[1]);
                },

                /* Méthode dédiée au parsage d'une ressource de type
                 * "GeometryCollection" issue d'une source GeoJSON.
                 */
                "GeometryCollection": function (data) {
                    var result = new Array(),
                        _this = this,
                        style = arguments[1];
                    // Pour chaque paquet de coordonnées, on génère les
                    // objets correspondants ...
                    $.each(data.geometries, function (i, _data) {
                        result.push(_this.parsing[_data.type].call(_this,
                            { coordinates: _data.coordinates }, style));
                    });
                    // On retourne enfin le fruit du parsage.
                    return result;
                },

                /* Méthode dédiée au parsage d'une ressource de type
                 * "Feature" issue d'une source GeoJSON.
                 */
                "Feature": function (data) {
                    // On tente de récupérer les potentielles propriétés 
                    // de style pour les appliquer sur le/les objets.
                    var style = typeof data.properties == "object" && data.properties != null &&
                        typeof data.properties.style == "object" && data.properties.style != null ?
                        data.properties.style : null;
                    // On commence par générer l'objet géométrique
                    // indiqué dans l'objet 'Feature'.
                    var geometry = this.parsing[data.geometry.type].call(this, data.geometry, style);
                    // Si l'objet dispose de propriétés permettant de
                    // le décrire, on les intègre par défaut dans celui-ci
                    // tout en offrant la possibilité d'intégrer une 
                    // popup à celui-ci !
                    if (typeof data.properties == "object" && data.properties != null) {
                        var appendProperties = function (geometry, data) {
                            geometry.properties = Lascar.prototype.fn.unfold(data.properties);
                            // Si la méthode de parsage des propriétés existe,
                            // on l'utilise !
                            if (typeof this.parsing["properties"] == "function") {
                                this.parsing["properties"].call(this, geometry);
                            }
                        };

                        // S'il s'agît d'un tableau d'objets géométriques
                        // on intègre les propriétés dans chacun d'entre eux,
                        if (geometry instanceof Array) {
                            var _this = this;
                            $.each(geometry, function (i, _geometry) {
                                appendProperties.call(_this, _geometry, data);
                            });
                        }
                        // sinon on les intègre directement dans l'objet.
                        else {
                            appendProperties.call(this, geometry, data);
                        }
                    }

                    // que l'on retourne enfin ...
                    return geometry;
                },

                /* Méthode dédiée au parsage d'une ressource de type
                 * "FeatureCollection" issue d'une source GeoJSON.
                 */
                "FeatureCollection": function (data) {
                    var result = new Array(),
                        _this = this;
                    // Pour chaque paquet de coordonnées, on génère les
                    // objets correspondants ...
                    $.each(data.features, function (i, feature) {
                        var geometries = _this.parsing["Feature"].call(_this, feature);
                        // Pour chaque élément parsé, on l'intègre dans
                        // le résultat, et dans le cas d'un tableau, on 
                        // intègre chacun des éléments.
                        if (geometries instanceof Array) {
                            $.each(geometries, function (i, geometry) {
                                result.push(geometry);
                            });
                        } else {
                            result.push(geometries);
                        }
                    });
                    // On retourne enfin le fruit du parsage.
                    return result;
                }

                /* Méthode dédié au parsage des propriétés d'une
                 * géométrie pour les intégrer dans une dialogue.
                 */
                // , "properties": Lascar.prototype.cl.layer.UrlBasedLayer.prototype._parseProperties

            }

        }),

        /* Classe permettant de générer un layer exploitant 
         * des ressources distantes issues par exemple de
         * web services retournant des données OverPass.
         */
        OverPass: Lascar.prototype.cl.layer.UrlBasedLayer.extend({

            /* Constructeur du layer.
             */
            initialize: function (reqConfig) {
                // On commence par appeller le constructeur
                // parent pour initialiser le layer ...
                Lascar.prototype.cl.layer.UrlBasedLayer.prototype.initialize.apply(this, arguments);

                // Puis on complète les réactions aux différents
                // statuts de la requête.
                //   - Dans le cas d'une réponse en succès
                //     on parse les données reçues.
                this.req.on("success", function (event) {
                    // On commence par purger le layer actuel ...
                    this.clearLayers();
                    try {
                        // La première étape consiste à référencer les différentes
                        // ressources en fonction de leur type : node, way, relation.
                        var overpassData = {},
                            _this = this,
                            geoObjects = new Array();
                        $.each(event.data.elements, function(i, elem) {
                            if(typeof overpassData[elem.type] != "object" || overpassData[elem.type] == null) {
                                overpassData[elem.type] = {};
                            }
                            overpassData[elem.type][elem.id] = elem;
                        });

                        // On peut enfin générer les éléments fournis par 
                        // le résultat de la requête.
                        $.map(overpassData, function(dataType, type) {
                            // Si une fonction de parsage existe pour le type,
                            // on traite les données associées !
                            if(typeof _this.parsing[type] == "function") {
                                $.map(dataType, function(elem, id) {
                                    var geometry = _this.parsing[type].call(overpassData, elem, _this.options.styles, _this);
                                    // Si la ressource dipose de tags, on les intègre si
                                    // possible au travers de la popup !
                                    if(typeof elem.tags == "object" && elem.tags != null && !(geometry instanceof Array)) {
                                        geometry.properties = Lascar.prototype.fn.unfold(elem.tags);
                                        // Si la fonction pour les propriétés existe,
                                        // on l'exploite !
                                        if(typeof _this.parsing["properties"] == "function") {
                                            _this.parsing["properties"].call(_this, geometry);
                                        }
                                    }

                                    if(geometry instanceof Array) { geoObjects.concat(geometry); }
                                    else { geoObjects.push(geometry); }
                                });
                            }
                        });

                        // On intègre enfin chacun des objets
                        $.each(geoObjects, function (i, geoObject) {
                            geoObject.addTo(_this);
                        });

                        this.notify(
                            "Parsage terminé !",
                            $("<i>", { class: "alert fa fa-thumbs-up" })[0]
                        );
                    } catch (e) {
                        this.notify(
                            "Erreur lors du parsage des données.",
                            $("<i>", { class: "alert fa fa-exclamation-triangle" })[0]
                        );
                    }
                }, this);
            },

            /* Méthode permettant de provoquer le chargement du 
             * layer à partir d'une requête et d'un secteur désigné
             * par une BBOX.
             */
            apply: function(query, bounds) {
                // On vérifie que les paramètres fournis sont valides :
                //  - query est bien une chaîne de caractères
                //  - bounds est bien soit un instance de L.LatLngBounds 
                //    soit un tableau de 4 nombres.
                if(typeof query != "string") {
                    throw new Error("The query must be a valid string.");
                }

                if(!(bounds instanceof L.LatLngBounds) && !Lascar.prototype.fn.check(bounds, "number[]") && bounds.length != 4) {
                    throw new Error("The bounds must be a valid LatLngBounds instance of an array of 4 numbers.");
                }

                // On transforme la BBOX en quelque chose d'exploitable,
                // surtout dans le cas d'une instance de LatLngBounds.
                var queryBounds = bounds;
                if(bounds instanceof L.LatLngBounds) {
                    queryBounds = [
                        parseFloat(bounds.getSouthWest().lat.toFixed(6)),
                        parseFloat(bounds.getSouthWest().lng.toFixed(6)),
                        parseFloat(bounds.getNorthEast().lat.toFixed(6)),
                        parseFloat(bounds.getNorthEast().lng.toFixed(6))
                    ];
                } 

                // Une fois que l'ensemble des éléments sont corrects,
                // on envoie la requête en ayant remplacé la mention "{{bbox}}"
                // par la zone fournie en paramètres.
                this.req.send({
                    data: { 
                        data: query.replace(/{{bbox}}/g, queryBounds.join(",")) 
                    }
                });
            },

            /* Méthodes de parsage des différentes ressources
             * gérées dans le format OverPass pour compléter le
             * layer actuel.
             */
            parsing: {

                /* Méthode dédiée au parsage d'une ressource de type
                 * "node" issue d'une source OverPass.
                 */
                "node": function (data) {
                    // Si des propriétés de style sont définies, on 
                    // les intègre, sinon on utilise celles par défaut.
                    var style = typeof arguments[1] == "object" && arguments[1] != null ?
                        arguments[1]["point"] : null;
                    // On génère enfin le marqueur ...
                    return new L.circleMarker(new L.LatLng(data.lat, data.lon), style);
                },

                /* Méthode dédiée au parsage d'une ressource de type
                 * "way" issue d'une source OverPass.
                 */
                "way": function (data) {
                    // Un chemin est considéré comme un polygone
                    // lorsqu'au moins un de ses points est traversé 
                    // plus d'une fois.
                    var isPolygon = $.map(data.nodes.count(), function (v, i) { if (v > 1) { return i; } }).length > 0;
                    // On récupère le style défini dans les options
                    // en fonction de s'il s'agît d'un polygone ou
                    // d'une ligne.
                    var style = typeof arguments[1] == "object" && arguments[1] != null ?
                        arguments[1][isPolygon ? "forme" : "ligne"] : null;
                    // on compile les coordonnées de l'ensemble des
                    // points composant le chemin ...
                    var coordinates = new Array(),
                        _this = this;
                    $.each(data.nodes, function (i, _n) {
                        var node = _this.node[_n];
                        if (typeof node != "undefined") {
                            coordinates.push([node.lat, node.lon]);
                        }
                    });
                    // et on génère la ligne ou le polygone associée !
                    return new (isPolygon ? L.polygon : L.polyline)(coordinates).setStyle(style);
                },

                /* Méthode dédiée au parsage d'une ressource de type
                 * "relation" issue d'une source OverPass.
                 */
                "relation": function (data) {
                    var geometries = new Array(),
                        parent = arguments[2],
                        _this = this;
                    // Pour chacun des membres de la relation, on
                    // le représente à l'aide des fonctions dédiés
                    // aux relations !
                    $.each(data.members, function(i, item) {
                        var function2use = null;
                        // On commence par tenter d'identifier la fonction
                        // qui va permettre le parsage de la donnée ...
                        if($.inArray(item.type, ["node", "way"]) != -1) {
                            if(typeof parent.parsing.relations[item.type] == "function") {
                                function2use = parent.parsing.relations[item.type];
                            }
                        } else {
                            if(typeof parent.parsing[item.type] == "function") {
                                function2use = parent.parsing[item.type];
                            }
                        }

                        // .. si la fonction existe, ainsi que la donnée,
                        // on traite leur import !
                        if(function2use != null && typeof _this[item.type][item.ref] != "undefined") {
                            geometries.push(function2use.call(
                                _this, 
                                _this[item.type][item.ref], 
                                parent.options.styles, 
                                parent));
                        }
                    });

                    return geometries;
                }

                /* Méthode dédié au parsage des propriétés d'une
                 * géométrie pour les intégrer dans une dialogue.
                 */
                // , "properties": Lascar.prototype.cl.layer.UrlBasedLayer.prototype._parseProperties

            }

        })

    });

    /* METHODES complémentaires de parsage pour les 
     * objets de type "node" ou "way" contenus dans
     * une relation. Ci-dessous, on se contente de 
     * réexploiter les fonctions de base.
     */
    Lascar.prototype.cl.layer.OverPass.prototype.parsing.relations = {
        "node": Lascar.prototype.cl.layer.OverPass.prototype.parsing.node,
        "way": Lascar.prototype.cl.layer.OverPass.prototype.parsing.way
    };


    /* Domaine permettant de contenir les classes 
     * de génération des composants pour une carte 
     * générée par Lascar.
     */
    Lascar.prototype.cl.component = {

        /* Classe permettant de générer une instance d'un gestionnaire
         * de paramètres dans l'URL impactant la/les cartes référencées.
         */
        Url: L.Evented.extend({

            /* Constructeur d'un gestionnaire de paramètres dans
             * l'URL pour les cartes observées.
             */
            initialize: function () {
                this.maps = new Lascar.prototype.cl.util.ArrayOf(
                    Lascar.prototype.cl.Map);
                this.markers = {};
                this.urlmarkersLayer = null;
                // On attache ce composant à l'événement "hashchange"
                // du composant UrlHashManager pour répecuter les 
                // changements dans l'URL sur les cartes.
                Lascar.prototype.UrlHashManager.on("hashchange", function (event) {
                    var data = this.unserialize(),
                        markers = {};
                    // On en profite pour traiter la génération 
                    // ou la mise-à-jour du layer en charge des 
                    // marqueurs.
                    $.map(data, function (subdata, key) {
                        if (Lascar.prototype.fn.check(subdata, Lascar.prototype.dc.D_MarkerConfig)) {
                            markers[key] = subdata;
                        }
                    });
                    this.buildMarkers(markers);

                    // On applique les changements une fois les marqueurs chargés.
                    this.fire("applychanges", data);
                }, this);
            },

            /* Méthode permettant de générer les marqueurs et les
             * associer au layer de base en charge de leur affichage
             * sur chacune des cartes liées à ce composant.
             */
            buildMarkers: function (data) {
                // Première étape, on commence par générer si nécessaire
                // le layer de base et dans le cas où celui-ci existe déjà,
                // on le purge de son contenu !
                if (this.urlmarkersLayer instanceof Lascar.prototype.cl.layer.FeaturesLayer) {
                    this.urlmarkersLayer.clearLayers();
                } else {
                    this.urlmarkersLayer = new Lascar.prototype.cl.layer.FeaturesLayer();
                }

                var _this = this;
                this.markers = {};
                // Pour chaque portion d'informations concernant
                // un marqueur, on l'intègre dans le layer de base.
                $.map(data, function (subdata, key) {
                    if (Lascar.prototype.fn.check(subdata, Lascar.prototype.dc.D_MarkerConfig)) {
                        var marker = null;
                        // S'il existe une icône pour le code associé,
                        // on l'utilise, sinon on utilise le marqueur
                        // par défaut.
                        if (Lascar.prototype.markers[subdata.marker] instanceof L.Icon) {
                            marker = new L.Marker(new L.LatLng(subdata.lat, subdata.lng), { icon: Lascar.prototype.markers[subdata.marker] });
                        } else {
                            marker = new L.Marker(new L.LatLng(subdata.lat, subdata.lng));
                        }

                        marker.addTo(_this.urlmarkersLayer);
                        _this.markers[key] = subdata;
                    }
                });

                // Pour chacune des cartes gérées par le gestionnaire de
                // paramètres dans l'URL, on intègre ce layer mis-à-jour.
                $.each(this.maps, function (i, map) {
                    _this.urlmarkersLayer.addTo(
                        "m_{0}__urlmarkers".format(map._leaflet_id),
                        map, "Marqueurs");
                });
            },

            /* Méthode permettant d'ajouter une carte Leaflet 
             * issue de Lascar au composant actuel. Cela initialise
             * aussi les comportements nécessaires au bon fonctionnement
             * du composant actuel.
             */
            addTo: function (map) {
                // On vérifie que la carte est bien une carte
                // Leaflet issue de Lascar !
                if (!(map instanceof Lascar.prototype.cl.Map)) {
                    throw new Error("Url needs a valid Lascar's map.");
                }

                // On référence la carte dans la liste des 
                // cartes gérées par le composant.
                this.maps.push(map);

                // On associe un comportement au moment d'un 
                // mouvement ou d'un changement de layer sur
                // la carte pour mieux répercuter ces changements
                // dans l'URL.
                map.on("moveend baselayerchange", function (event) {
                    this.serialize();
                }, this);

                this.on("applychanges", function (event) {
                    var mapId = $(this._container).attr("id"),
                        mapConfig = event[mapId];

                    // S'il existe un paramètre pour une configuration
                    // associé au code de la carte, et que celle-ci est
                    // valide structurellement, on l'exploite !
                    if (typeof mapConfig == "object" && mapConfig != null &&
                        Lascar.prototype.fn.check(mapConfig, Lascar.prototype.dc.D_MapConfig)) {
                        this.apply(mapConfig);
                    }
                }, map);

                // On retourne enfin l'objet actuel !
                return this;
            },

            /* Méthode permettant d'intégrer dans l'URL les paramètres
             * associés aux cartes gérées par le composant.
             */
            serialize: function () {
                var urlHash = new Array();

                // Pour chaque carte, on récupère sa configuration
                // que l'on va intégrer dans l'URL.
                $.each(this.maps, function (i, map) {
                    urlHash.push(
                        "{0}={1}/{2}/{3},layer={4}".format(
                            $(map._container).attr("id"),
                            map.getZoom(),
                            map.getCenter().lat,
                            map.getCenter().lng,
                            map.getActualBaseLayer().options.code)
                    );
                });

                // Pour chaque marqueur référencé, on l'intègre
                // dans l'URL ...
                $.each(this.markers, function (key, marker) {
                    urlHash.push(
                        "{0}={1}/{2}/{3}".format(
                            key,
                            marker.lat,
                            marker.lng,
                            marker.marker)
                    );
                });

                // On intègre la nouvelle valeur du hash à partir
                // de la sérialization des configurations des cartes
                // et marqueurs observées.
                Lascar.prototype.UrlHashManager.goto(urlHash.join("&"));
                Lascar.prototype.UrlHashManager._isProgrammatic = false;
            },

            /* Méthode permettant de parser les paramètres de
             * l'URL afin d'en extraire les configurations des
             * différents composants.
             */
            unserialize: function () {
                var data = {};

                // Si l'URL contient des éléments ...
                if ($.trim(global.location.hash.substr(1)) != "") {
                    // Pour chaque élément de données dans l'URL, on
                    // le traite de manière à en extraire la configuration
                    // des éléments gérés par ce composant.
                    $.each(global.location.hash.substr(1).split("&"), function (i, d) {
                        // Si la donnée est de la forme (id)=(...) 
                        // on peut envisager de la traiter, sinon
                        // on part du principe que la valeur associée
                        // vaut 'null' ...
                        if (d.match(/([a-zA-Z0-9_-]+)=(.*)/)) {
                            // Pour traiter correctement la donnée
                            // associée à la clé, il nous faut identifier
                            // s'il s'agît de données cartographiques ou
                            // de marqueurs.
                            var sepPosition = d.indexOf("="),
                                dataKey = d.substr(0, sepPosition),
                                dataValue = d.substr(++sepPosition),
                                dataValueMap = undefined;
                            // S'il s'agît de données cartographiques on
                            // les traite en tant que tel !
                            if ((dataValueMap = dataValue.match(/(-?[0-9]*\.?[0-9]*)\/(-?[0-9]*\.?[0-9]*)\/(-?[0-9]*\.?[0-9]*),layer=(.+)/))) {
                                data[dataKey] = {
                                    zoom: parseInt(dataValueMap[1]),
                                    lng: parseFloat(dataValueMap[3]),
                                    lat: parseFloat(dataValueMap[2]),
                                    layer: dataValueMap[4]
                                };
                            }
                            // S'il s'agît de données de marqueurs on
                            // les traite en tant que tel !
                            else if ((dataValueMap = dataValue.match(/(-?[0-9]*\.?[0-9]*)\/(-?[0-9]*\.?[0-9]*)\/([a-zA-Z0-9_-]+)/))) {
                                data[dataKey] = {
                                    lng: parseFloat(dataValueMap[2]),
                                    lat: parseFloat(dataValueMap[1]),
                                    marker: dataValueMap[3]
                                };
                            }
                            // Sinon on ne traite pas la donnée ...
                        } else {
                            data[d] = null;
                        }
                    });
                }

                return data;
            }

        }),

        /* Classe permettant de générer une instance d'un requêteur 
         * à partir de paramètres permettant de configurer la requête
         * avant chaque envoi.
         */
        Requester: L.Evented.extend({
            /* Paramètre permettant de définir les options
             * par défaut du requêteur.
             */
            default: {
                // Communication asynchrone avec le web service.
                async: true,
                // Non mise-en-cache des données reçues.
                cache: false,
                // Refuser le requêtage inter-domaines.
                crossDomain: false,
                // Contenu de type JSON.
                contentType: "application/json",
                // Parsage de la réponse au format JSON.
                dataType: "json",
                // En-têtes à ajouter à la requête.
                headers: {}
            },

            /* Constructeur d'un requêteur, permettant d'initialiser
             * ses options à partir de celles fournies par défaut.
             */
            initialize: function (config) {
                this.options = Lascar.prototype.fn.extend(this.default, config);
            },

            /* Méthode permettant d'envoyer la requête si celle-ci est
             * bien configurée, tout en offrant la possibilité à
             * l'émetteur de modifier les options avant l'envoi.
             */
            send: function (config) {
                // On intègre les potentielles corrections de 
                // configuration des options avant d'émettre 
                // la requête.
                if (typeof config == "object" && config != null) {
                    this.options = Lascar.prototype.fn.extend(this.options, config);
                }

                // On vérifie que l'on dispose des informations
                // nécessaires pour générer la requête sans quoi
                // on retourne une erreur !
                if (!Lascar.prototype.fn.check(this.options, Lascar.prototype.dc.D_Requester)) {
                    throw new Error("Requester needs a valid configuration.");
                }

                var ajaxOptions = $.extend(true, {}, this.options);
                // S'il faut prendre en compte des en-têtes, on
                // génère la fonction appropriée pour jQuery.
                if (typeof ajaxOptions.headers == "object" && ajaxOptions.headers != null) {
                    ajaxOptions.beforeSend = function (xhr) {
                        $.map(ajaxOptions.headers, function (value, key) {
                            xhr.setRequestHeader(key, value);
                        });
                    };
                }

                // On averti les observateurs que l'on vient
                // d'envoyer la requête.
                this.fire("sending");

                var _this = this,
                    result = null;
                // On peut enfin emmettre la requête !
                $.ajax(ajaxOptions)
                    .done(function (data, status, jqXHR) {
                        _this.fire("success", (result = {
                            "data": data,
                            "status": status,
                            "jqXHR": jqXHR
                        }));
                    }).fail(function (jqXHR, status, error) {
                        _this.fire("fail", (result = {
                            "error": error,
                            "status": status,
                            "jqXHR": jqXHR
                        }));
                    });

                // S'il s'agît d'un appel asynchrone, on retourne
                // la réponse du web service directement ...
                if (!(typeof this.options.async == "boolean" ? this.options.async : true)) {
                    return result;
                }
            }

        }),

        /* Classe permettant de générer un composant en charge de
         * l'affichage de notifications sur la carte à laquelle il
         * est associé.
         */
        Notifier: L.Control.extend({

            /* Paramètre permettant de définir les options
             * par défaut pour la génération d'un message
             * de notification.
             */
            default: {
                icon: $("<span>", {
                    class: "alert", html: " ! ",
                    css: { "color": "red", "font-weight": "bold" }
                })[0],
                message: "Un événement est survenu ..."
            },

            /* Options du contrôle de notifications.
             */
            options: {
                // Position par défaut : en haut à gauche.
                position: "topleft",
                // Nombre maximum de messages dans le composant
                maxMessages: 5
            },

            /* Méthode permettant de traiter la génération du 
             * composant suite à son intégration dans une carte 
             * Leaflet.
             */
            onAdd: function (m) {
                var _this = this;
                // On génère le conteneur du "notifier"
                this.container = L.DomUtil.create("div", "leaflet-control lascar-leaflet-notifier");
                // On défini la taille du conteneur ...
                $(this.container).css("width", $(m._container).width()*.75);
                // On ajoute un bouton permettant de fermer le composant
                $("<a>", {
                    href: "#",
                    html: "&times;",
                    class: "lascar-notifier-close",
                    title: "Fermer"
                }).click(function (event) {
                    // On masque le composant ...
                    _this.hide();
                    // ... et on en profite pour le purger !
                    $(_this.container).find(".message").remove();
                    event.preventDefault();
                }).appendTo($(this.container));
                // ... on le masque par défaut et on le retourne enfin.
                return (this.hide()).container;
            },

            /* Méthode permettant d'intégrer un message dans 
             * le composant de notifications.
             */
            _integrateMessage: function (params) {
                var $container = $(this.container);
                // S'il y a plus de messages que permis, on supprime
                // le plus ancien pour permettre l'intégration du
                // nouveau message de notification.
                if ($container.find(".message").length >= this.options.maxMessages) {
                    while ($container.find(".message").length >= this.options.maxMessages) {
                        $container.find(".message").first().remove();
                    }
                }

                // Pour tous les messages actuellement en place, on stop
                // l'animation de l'icône actuelle !
                $container.find(".message .notifier-icon").children().addClass("no-anim");

                // On peut dorénavant intégrer le nouveau message !
                var $ctMessage = $("<div>", { class: "message" });
                var $ctIcon = $("<p>", { class: "notifier-icon" }).appendTo($ctMessage);

                // On intègre l'icône fournie dans la notification.
                if (typeof params.icon != "undefined") {
                    $(params.icon).appendTo($ctIcon);
                }

                // On intègre le message dans la notification.
                if (typeof params.message == "string" && $.trim(params.message) != "") {
                    $("<p>", { html: params.message }).appendTo($ctMessage);
                }

                $ctMessage.appendTo($container);
            },

            /* Méthode permettant d'afficher le composant en
             * charge des notifications à l'aide des informations
             * fournies en paramètre.
             */
            show: function (params) {
                // Si le paramètre est un objet, on ajoute le message
                // à la pile dans la mesure du possible.
                if (typeof params == "object" && params != null) {
                    this._integrateMessage(params);
                }
                // Si le paramètre est un tableau, on ajoute les message
                // dans la pile dans la mesure du possible.
                else if (Lascar.prototype.fn.check(params, "object[]")) {
                    var _this = this;
                    $.each(params, function (i, _param) {
                        _this._integrateMessage(_param);
                    });
                }
                // Si aucun paramètre n'est fourni, on intègre le message
                // par défaut dans la pile dans la mesure du possible.
                else {
                    this._integrateMessage(this.default);
                }

                // On affiche enfin le composant !
                $(this.container).show();

                // et enfin on retourne le composant.
                return this;
            },

            /* Méthode permettant de masquer le composant en
             * charge de l'affichage des notifications.
             */
            hide: function () {
                // On masque le conteneur ...
                $(this.container).hide();
                // et enfin on retourne le composant.
                return this;
            }
        })

    };

    /* Classe permettant de générer une carte Leaflet
     * dîte classique tout en profitant des fonctionnalités
     * apportées par Lascar.
     */
    Lascar.prototype.cl.Map = L.Map.extend({
        /* Classe permettant de définir les options
         * par défaut de la carte Leaflet.
         */
        Default: L.Class.extend({

            initialize: function () {
                // Identifiant par défaut de la carte
                this.id = "lf-map";

                // Options par défaut de la carte
                this.options = {
                    // Style par défaut de la carte
                    "style": {
                        "width": "100%",
                        "height": "100%"
                    },
                    // Centre de la carte
                    "center": new L.LatLng(46.53972, 2.43028),
                    // Niveau de zoom
                    "zoom": 6,
                    // Contrôle ZOOM (true|false|<Zoom options>)
                    "zoomControl": {
                        "position": "topright",
                        "zoomInTitle": "Zoomer",
                        "zoomOutTitle": "Dézoomer"
                    },
                    // Contrôle SCALE (true|false[<Scale options>)
                    "scaleControl": {
                        "position": "bottomright"
                    },
                    "base_layers": new Lascar.prototype.cl.util.ArrayOf(
                        Lascar.prototype.cl.layer.RasterTileLayer, [
                        new Lascar.prototype.cl.layer.RasterTileLayer(
                            "http://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png", {
                                "code": "osm_basic",
                                "title": "OpenStreetMap",
                                "attribution": '© Contributeurs d\'<a href="http://openstreetmap.org">OpenStreetMap</a> - Hébergeur : <a href="http://openstreetmap.org/">OpenStreetMap</a>',
                                "maxZoom": 19,
                                "default": true
                            }),
                        new Lascar.prototype.cl.layer.RasterTileLayer(
                            "http://{s}.tile.openstreetmap.fr/osmfr/{z}/{x}/{y}.png", {
                                "code": "osm_fr",
                                "title": "OpenStreetMap (FR)",
                                "attribution": '© Contributeurs d\'<a href="http://openstreetmap.org">OpenStreetMap</a> - Hébergeur : <a href="http://tile.openstreetmap.fr/">OpenStreetMap (FR)</a>'
                            })
                    ])
                };
            }
        }),

        /* Constructeur d'une carte Leaflet permettant de
         * la générer avec des options personnalisées par
         * l'utilisateur : ID et/ou options ou la configuration
         * par défaut.
         */
        initialize: function () {
            // NOTE :
            // Par rapport aux options, en ce qui concerne le
            // contrôle de type Zoom, si l'option défini un objet,
            // on commence par ne pas le générer pour avoir toute
            // la latitude par la suite.
            var zoomControl = false,
                _id = undefined,
                _options = undefined;

            // On instancie les paramètres par défaut.
            this.default = new Lascar.prototype.cl.Map.prototype.Default();

            if (arguments.length == 0) {
                if (typeof this.default.options.zoomControl == "object" &&
                    this.default.options.zoomControl != null) {
                    zoomControl = this.default.options.zoomControl;
                    this.default.options.zoomControl = false;
                }

                _id = this.default.id;
                _options = this.default.options;
            } else if (arguments.length == 1) {
                var custom = arguments[0];
                // Si l'argument est une chaîne de caractères,
                // l'utilisateur souhaite personnaliser uniquement
                // l'identifiant de la carte.
                if (typeof custom == "string" && $.trim(custom) != "") {
                    if (typeof this.default.options.zoomControl == "object" &&
                        this.default.options.zoomControl != null) {
                        zoomControl = this.default.options.zoomControl;
                        this.default.options.zoomControl = false;
                    }

                    _id = custom;
                    _options = this.default.options;
                }
                // Si l'argumetnt passé est un objet non null, l'utilisateur
                // souhaite personnaliser uniquement les options.    
                else if (typeof custom == "object" && custom != null) {
                    var extendedOptions = Lascar.prototype.fn.extend(this.default.options, custom);
                    if (typeof extendedOptions.zoomControl == "object" &&
                        extendedOptions.zoomControl != null) {
                        zoomControl = extendedOptions.zoomControl;
                        extendedOptions.zoomControl = false;
                    }

                    _id = this.default.id;
                    _options = extendedOptions;
                }
                // S'il ne s'agît aucun des cas ci-dessus, on retourne
                // une erreur pour notifier l'utilisateur que le paramètre
                // fourni est incorrect.
                else {
                    throw new Error("Map needs a valid string or object parameter.");
                }
            } else {
                var id = arguments[0],
                    opts = arguments[1];
                // Si les paramètres sont valides, on les intègre
                if (typeof id == "string" && $.trim(id) != "" && typeof opts == "object" && opts != null) {
                    var extendedOptions = Lascar.prototype.fn.extend(this.default.options, opts);
                    if (typeof extendedOptions.zoomControl == "object" &&
                        extendedOptions.zoomControl != null) {
                        zoomControl = extendedOptions.zoomControl;
                        extendedOptions.zoomControl = false;
                    }

                    _id = id;
                    _options = extendedOptions;
                }
                // Sinon on retourne une erreur.
                else {
                    throw new Error("Map needs a valid string and object parameters.");
                }
            }

            // On applique la potentielle configuration de style.
            this._initStyle(_id, _options);

            // On génère la carte !
            L.Map.prototype.initialize.call(this, _id, _options);

            // On génère le conteneur pour référencer les layers
            // complémentaires ...
            this.options.cpl_layers = {};

            // Si des layers de base sont définis, on les applique.
            if (Lascar.prototype.fn.check(this.options.base_layers,
                Lascar.prototype.cl.util.ArrayOf,
                Lascar.prototype.cl.layer.RasterTileLayer)) {

                var defaultLayer = undefined,
                    // Pour chaque layer de la configuration, 
                    // on less référence dans un objet ...
                    layersToAppend = {};
                $.each(this.options.base_layers, function (i, layer) {
                    layersToAppend[layer.options.title] = layer;
                    // S'il s'agît d'un layer par défaut, on le
                    // référence de manière à obtenir à la fin le
                    // dernier des layer par défaut.
                    if (layer.options.default) { defaultLayer = layer; }
                });
                // ... pour ensuite générer le contrôle approprié.
                (this.layersControl = new L.Control.Layers(layersToAppend))
                    .addTo(this);
                // Si un layer par défaut est défini, on le
                // référence en tant que tel.
                if (typeof defaultLayer != "undefined") {
                    this.addLayer(defaultLayer);
                }
            }

            // Si l'option "zoomControl" (sauvegardée) est bien 
            // un objet, on génère le contrôle customisé !
            if (typeof zoomControl == "object" && zoomControl != null) {
                this._generateZoomControl(zoomControl);
            }

            // Si l'option "scaleControle" un objet, ou
            // vaut "true", on génère le contrôle 
            // potentiellement customisé !
            if ((typeof this.options.scaleControl == "boolean" && this.options.scaleControl) ||
                (typeof this.options.scaleControl == "object" && this.options.scaleControl != null)) {
                this._generateScaleControl(typeof this.options.scaleControl == "object" ? this.options.scaleControl : null);
            }
        },

        /* Méthode permettant d'initialiser le style associé
         * au conteneur de la carte si celui-ci est défini.
         */
        _initStyle: function (id, opts) {
            // Si un style est défini, on l'applique !
            if (typeof opts.style == "object" && opts.style != null) {
                $("#{0}".format(id)).css(opts.style);
            }
        },

        /* Méthode permettant de générer le contrôle en charge
         * du Zoom sur la carte. S'il existe déjà, l'ancien est 
         * supprimé pour pemrettre la génération du nouveau.
         */
        _generateZoomControl: function (opts) {
            // Si le contrôle est déjà défini, on le supprime
            // pour mieux le regénérer !
            if (this.zoomControl instanceof L.Control.Zoom) {
                this.removeControl(this.zoomControl);
            }
            // Enfin on peut régénérer le contrôle !
            (this.zoomControl = new L.Control.Zoom(opts)).addTo(this);
        },

        /* Méthode permettant de générer le contrôle en charge
         * du de l'échelle sur la carte. S'il existe déjà, l'ancien  
         * est supprimé pour pemrettre la génération du nouveau.
         */
        _generateScaleControl: function (opts) {
            // Si le contrôle est déjà défini, on le supprime
            // pour mieux le regénérer !
            if (this.scaleControl instanceof L.Control.Scale) {
                this.removeControl(this.scaleControl);
            }
            // Enfin on peut régénérer le contrôle !
            (this.scaleControl = new L.Control.Scale(opts)).addTo(this);
        },

        /* Méthode permettant de retourner le layer actuellement
         * utilisé par la carte (si disponible).
         */
        getActualBaseLayer: function () {
            var actualBaseLayer = null,
                _this = this;
            // Pour chaque layer actuellement attaché à la carte,
            // on recherche celui qui n'est pas un "overlay".
            $.each(this.options.base_layers, function (i, cLayer) {
                if (cLayer.options.pane == "tilePane" && _this.hasLayer(cLayer)) { actualBaseLayer = cLayer; }
            });
            return actualBaseLayer;
        },

        /* Méthode permettant de récupérer la liste des layers
         * actuellement utilisés par la carte et n'étant pas 
         * des layers de base.
         */
        getActualOverlayLayers: function () {
            var actualOverlayLayer = new Array();
            // Pour chaque layer actuellement attaché à la carte,
            // on recherche celui qui n'est pas de "base".
            $.each(this._layers, function (i, cLayer) {
                if (cLayer.options.pane != "tilePane") { actualOverlayLayer.push(cLayer); }
            });
            return actualOverlayLayer;
        },

        /* Méthode permettant de récupérer un layer de base
         * attaché à la carte à partir de son code.
         */
        getBaseLayer: function (code) {
            var layer = null;
            $.each(this.options.base_layers, function (i, baseLayer) {
                if (baseLayer.options.code == code && baseLayer instanceof Lascar.prototype.cl.layer.RasterTileLayer) {
                    layer = baseLayer;
                }
            });
            return layer;
        },

        /* Méthode permettant de modifier l'affichage de la carte
         * soit en restaurant son affichage à celui par défaut, 
         * soit en utilisant la configuration fournie en paramètre.
         */
        apply: function () {
            var params = arguments[0] || {},
                // On détermine le mode d'application souhaité 
                // à partir du paramètre fourni.
                applyMode = 0;

            // Si le paramètre dispose d'une information
            // sur le layer ...
            if (typeof params.layer == "string") { applyMode = 1; }
            // Si le paramètre dispose des informations 
            // sur les coordonnées du nouveau centre ...
            if (typeof params.lat == "number" && typeof params.lng == "number") { applyMode += 2; }

            // On récupère les paramètres par défaut afin
            // de le coupler avec les éléments fournis dans
            // le paramètre.
            var center = applyMode >= 2 ? new L.LatLng(params.lat, params.lng) : this.default.options.center,
                zoom = applyMode >= 2 ? params.zoom : this.default.options.zoom,
                layer = applyMode >= 1 ? params.layer : this.getActualBaseLayer().options.code;

            // Si le centre a changé, ou que le niveau de 
            // zomm est différent, on applique les nouvelle
            // informations !
            if (!this.getCenter().equals(center) || this.getZoom() != zoom) {
                this.setView(center, zoom);
            }

            var layerToApply = this.getBaseLayer(layer);
            // Si le layer de base a changé, on applique le nouveau 
            if (this.getActualBaseLayer().options.code != layer && layerToApply != null) {
                this.removeLayer(this.getActualBaseLayer());
                this.addLayer(layerToApply);
            }
        }

    });



    // ----------
    // PATTERNS
    // ----------

    /* Domaine permettant de contenir les patterns de validation
     * apportés par la librairie.
     */
    Lascar.prototype.dc = {};

    /* Pattern de validation d'une configuration complète 
     * pour une carte.
     */
    Lascar.prototype.dc.D_MapConfig = [
        new Lascar.prototype.cl.util.Description("zoom", "number"),
        new Lascar.prototype.cl.util.Description("lng", "number"),
        new Lascar.prototype.cl.util.Description("lat", "number"),
        new Lascar.prototype.cl.util.Description("layer", "string")
    ];

    /* Pattern de validation d'une configuration complète
     * pour un marqueur positionné sur la/les cartes.
     */
    Lascar.prototype.dc.D_MarkerConfig = [
        new Lascar.prototype.cl.util.Description("lng", "number"),
        new Lascar.prototype.cl.util.Description("lat", "number"),
        new Lascar.prototype.cl.util.Description("marker", "string")
    ];

    /* Pattern de validation d'une configuration d'un 
     * requêteur de services. 
     */
    Lascar.prototype.dc.D_Requester = [
        new Lascar.prototype.cl.util.Description("url", "string"),
        new Lascar.prototype.cl.util.Description("type", "string")
    ];


    // ----------
    // MARQUEURS
    // ----------

    /* Domaine permettant de référencer les marqueurs exploitables
     * par la librairie et entre autre le gestionnaire d'URLs.
     */
    Lascar.prototype.markers = {};



    // ----------
    // INSTANCES
    // ----------
    /* Instance permettant de gérer la partie "hash"
     * de l'URL de la fe^tre actuelle et ainsi distinguer
     * les changements issus de la programmation des autres.
     */
    Lascar.prototype.UrlHashManager = new (L.Evented.extend({

        /* Constructeur permettant d'initialiser le composant
         * en charge de la gestion du "hash" de l'URL de la 
         * fenêtre actuelle.
         */
        initialize: function () {
            this._isProgrammatic = false;
            // On attache le comportement approprié
            // à la fenêtre concernant l'événement "onhashchange".
            global.onhashchange = function (event) {
                var manager = Lascar.prototype.UrlHashManager;
                if (manager._isProgrammatic) {
                    manager.fire("proghashchange", event);
                    manager._isProgrammatic = false;
                } else {
                    manager.fire("hashchange", event);
                }
            };
        },

        /* Méthode permettant de modifier le "hash" de l'URL
         * de manière programmée. 
         */
        goto: function (hash) {
            this._isProgrammatic = true;
            global.location.hash = hash;
        }

    }));

    // S'il s'agît d'un navigateur, on initialise une
    // instance de la librairie.
    if (!noGlobal) {
        global.Lascar = new Lascar();
    }

    return Lascar;
});